from lib.database.entities.trust_trace import TrustTrace
from lib.database.entities.user_trust import UserTrust
from lib.database.entities.user import User
from lib.database.entities.trust_adapter import TrustAdapter
from lib.trust.message_handler import MessageHandler
from typing import List, Dict
import unittest
import json


class DummyTrustAdapterRepo:
    _adapters: List[TrustAdapter] = [
        TrustAdapter(
            "car-sharing",
            "context1",
            "travel",
            "user",
            "cost",
            "rating",
            1,
            5,
            2000,
            0.75,
            8,
            1,
            1
        )
    ]

    def add(self, adapter: TrustAdapter):
        self._adapters.append(adapter)

    def all(self) -> List[Dict]:
        return self._adapters

    def all_for_use_case_and_context(self, use_case: str, context: str) -> List[Dict]:
        return [adapter.to_serializable_dict() for adapter in self._adapters if (adapter.use_case == use_case and adapter.context == context)]

    def all_for_use_case_and_table(self, use_case: str, table: str) -> List[Dict]:
        return [adapter.to_serializable_dict() for adapter in self._adapters if (adapter.use_case == use_case and adapter.table == table)]

    def one_for_use_case_and_context_and_table(self, use_case: str, context: str, table: str) -> Dict:
        result = [adapter for adapter in self._adapters if (
            adapter.use_case == use_case and adapter.context == context and adapter.table == table)]

        if len(result) != 1:
            return None

        return result[0].to_serializable_dict()


class DummyUserRepo:
    _users: List[User] = []

    def clear(self):
        self._users.clear()

    def add(self, user: User):
        self._users.append(user)

    def all(self) -> List[User]:
        return self._users

    def all_for_use_case(self, use_case: str) -> List[User]:
        return [user for user in self._users if (user.use_case == use_case)]

    def one_by_use_case_and_username(self, use_case: str, username: str) -> User:
        result = [user for user in self._users if (
            user.use_case == use_case and user.name == username)]

        if len(result) != 1:
            return None

        return result[0]


class DummyUserTrustRepo:
    _user_trusts: List[UserTrust] = []

    def add(self, user_trust: UserTrust):
        self._user_trusts.append(user_trust)

    def all(self) -> List[UserTrust]:
        return self._user_trusts

    def all_for_use_case(self, use_case: str) -> List[UserTrust]:
        return [user_trust for user_trust in self._user_trusts if (user_trust.use_case == use_case)]

    def all_for_user(self, user: User) -> List[UserTrust]:
        return [user_trust for user_trust in self._user_trusts if (user_trust.use_case == user.use_case and user_trust.user == user.name)]

    def one_for_user_and_context(self, user: User, context: str) -> UserTrust:
        result = [user_trust for user_trust in self.user_trust if (
            user_trust.use_case == user.use_case and user_trust.name == user.name and user_trust.context == context)]

        if len(result) != 1:
            return None

        return result[0]

    def update(self, user_trust: UserTrust):
        pass


class DummyTrustTraceRepository:
    _traces: List[TrustTrace] = []

    def add(self, trace: TrustTrace):
        self._traces.append(trace)

    def all(self) -> List[Dict]:
        return [trace.to_serializable_dict() for trace in self._traces]

    def all_for_use_case_and_context(self, use_case: str, context: str) -> List[Dict]:
        result = [trace for trace in self._traces if (
            trace.use_case == use_case and trace.context == context)]

        return [trace.to_serializable_dict() for trace in result]


class Test_Messages(unittest.TestCase):

    user_repo = None
    message_handler: MessageHandler = None

    def setUp(self):
        self.user_repo = DummyUserRepo()
        self.message_handler = MessageHandler(
            DummyTrustAdapterRepo(),
            self.user_repo,
            DummyUserTrustRepo(),
            DummyTrustTraceRepository()
        )

    def _buildTrace(self):
        return json.dumps({
            "type": "new-trace",
            "content": {
                "use_case": "car-sharing",
                "table": "travel",
                "id": "092870924809481f",
                "properties": {
                    "user": "Bob",
                    "cost": 201.12,
                    "rating": 4
                }
            }
        })

    def test_message_no_json(self):
        message: str = "abcdefg"

        try:
            self.message_handler.handle(message)
        except ValueError:
            pass

    def test_message_valid(self):
        message = self._buildTrace()

        self.user_repo.clear()
        self.assertEqual(0, len(self.user_repo.all()))

        self.message_handler.handle(message)

        self.assertEqual(1, len(self.user_repo.all())) # ensure user was added


if __name__ == '__main__':
    unittest.main()
